#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"
#include "Clean.h"

Clean::Clean(int cleanPin, int buttonPin, int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalOutRow = 1;
  _muxShieldDigitalInRow = 2;
  _countControlCodes = 2;
  _initialCleanMode = cleanCleaner;
  _cleanRelay = new MyRelay(cleanPin, muxShield, _muxShieldDigitalOutRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setCleanMode(_initialCleanMode);
}

int Clean::setCleanMode(int cleanMode) {
  switch (cleanMode) {
  case cleanSkimmer:
    _cleanRelay->setRelay(0);
    break;
  case cleanCleaner:
    _cleanRelay->setRelay(1);
    break;
  default:
    return 1;
  }
  _control->setControlCode(cleanMode);
  _cleanMode = (CleanModes)cleanMode;
  return 0;
}

int Clean::getCleanMode() {
  return _cleanMode;
}

int Clean::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setCleanMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void Clean::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int Clean::getProcessing() {
  return _control->getProcessing();
}

int Clean::getButtonState() {
  return _button->getDebounceButtonState();
}

int Clean::getControlType() {
  return _control->getControlType();
}

int Clean::nextControlCode() {
  return _control->nextControlCode();
}

int Clean::getControlCode() {
  return _control->getControlCode();
}

char *Clean::getHeader() {
  char * buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  strcpy(buffer, "Clean Control");
  return buffer;
}

char *Clean::getControlText() {
  char * buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case cleanSkimmer:
    strcpy(buffer, "Skimmer");
    return buffer;
    break;
  case cleanCleaner:
    strcpy(buffer, "Cleaner");
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int Clean::getLastResult() {
  return _lastResult;
}
