/*
  Clean.h - A library for controlling the whether we are
            cleaning with a robot or skimming the pool
  Created by Ken Rumer, May 13, 2014.
  Released into the public domain.
*/
#ifndef Clean_h
#define Clean_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"

enum CleanModes {
  cleanSkimmer,
  cleanCleaner
};

class Clean
{
  public:
    Clean(int cleanPin, int buttonPin, int controlType, MuxShield *muxShield);
    int setCleanMode(int cleanMode);
    int getCleanMode();
    int controlDevice();
    void setProcessing(int processing);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalOutRow;
    int _muxShieldDigitalInRow;
    int _countControlCodes;
    CleanModes _initialCleanMode;
    MyRelay *_cleanRelay;
    MyButton *_button;
    MyControl *_control;
    CleanModes _cleanMode;
};
#endif
