#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"
#include "Flow.h"

Flow::Flow(int suctionPin, int outputPin, int buttonPin, int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalOutRow = 1;
  _muxShieldDigitalInRow = 2;
  _countControlCodes = 2;
  _initialFlowMode = flowPool;
  _suctionRelay = new MyRelay(suctionPin, muxShield, _muxShieldDigitalOutRow);
  _outputRelay = new MyRelay(outputPin, muxShield, _muxShieldDigitalOutRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setFlowMode(_initialFlowMode);
}

int Flow::setFlowMode(int flowMode) {
  switch (flowMode) {
  case flowPool:
    _suctionRelay->setRelay(flowMode);
    _outputRelay->setRelay(flowMode);
    break;
  case flowSpa:
    _suctionRelay->setRelay(flowMode);
    _outputRelay->setRelay(flowMode);
    break;
  default:
    return 1;
  }
  _control->setControlCode(flowMode);
  _flowMode = (FlowModes)flowMode;
  return 0;
}

int Flow::getFlowMode() {
  return _flowMode;
}

int Flow::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setFlowMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void Flow::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int Flow::getProcessing() {
  return _control->getProcessing();
}

int Flow::getButtonState() {
  return _button->getDebounceButtonState();
}

int Flow::getControlType() {
  return _control->getControlType();
}

int Flow::nextControlCode() {
  return _control->nextControlCode();
}

int Flow::getControlCode() {
  return _control->getControlCode();
}

char *Flow::getHeader() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  strcpy(buffer, "Flow Control");
  return buffer;
}

char *Flow::getControlText() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case flowPool:
    strcpy(buffer, "Pool");
    return buffer;
    break;
  case flowSpa:
    strcpy(buffer, "Spa");
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int Flow::getLastResult() {
  return _lastResult;
}
