/*
  Flow.h - A library for controlling the flow of water for a pool
  Created by Ken Rumer, May 13, 2014.
  Released into the public domain.
*/
#ifndef Flow_h
#define Flow_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"

enum FlowModes {
  flowPool,
  flowSpa
};

class Flow
{
  public:
    Flow(int suctionPin, int outputPin, int buttonPin, int controlType, MuxShield *muxShield);
    int setFlowMode(int flowMode);
    int getFlowMode();
    int controlDevice();
    void setProcessing(int controlCode);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalOutRow;
    int _muxShieldDigitalInRow;
    int _countControlCodes;
    FlowModes _initialFlowMode;
    MyRelay *_suctionRelay;
    MyRelay *_outputRelay;
    MyButton *_button;
    MyControl *_control;
    FlowModes _flowMode;
};
#endif
