#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"
#include "Heat.h"

Heat::Heat(int furnacePin1, int furnacePin2, int solarPin, int buttonPin,
           int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalOutRow = 1;
  _muxShieldDigitalInRow = 2;
  _countControlCodes = 4;
  _initialHeatMode = heatOff;
  _furnaceRelay1 = new MyRelay(furnacePin1, muxShield, _muxShieldDigitalOutRow);
  _furnaceRelay2 = new MyRelay(furnacePin2, muxShield, _muxShieldDigitalOutRow);
  _solarRelay = new MyRelay(solarPin, muxShield, _muxShieldDigitalOutRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setHeatMode(_initialHeatMode);
}

int Heat::setHeatMode(int heatMode) {
  switch (heatMode) {
  case heatOff:
    _furnaceRelay1->setRelay(HIGH);
    _furnaceRelay2->setRelay(HIGH);
    _solarRelay->setRelay(HIGH);
    break;
  case heatFurnace:
    _furnaceRelay1->setRelay(LOW);
    _furnaceRelay2->setRelay(LOW);
    _solarRelay->setRelay(HIGH);
    break;
  case heatSolar:
    _furnaceRelay1->setRelay(HIGH);
    _furnaceRelay2->setRelay(HIGH);
    _solarRelay->setRelay(LOW);
    break;
  case heatBoth:
    _furnaceRelay1->setRelay(LOW);
    _furnaceRelay2->setRelay(LOW);
    _solarRelay->setRelay(LOW);
    break;
  default:
    return 1;
  }
  _control->setControlCode(heatMode);
  _heatMode = (HeatModes)heatMode;
  return 0;
}

int Heat::getHeatMode() {
  return _heatMode;
}

int Heat::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setHeatMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void Heat::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int Heat::getProcessing() {
  return _control->getProcessing();
}

int Heat::getButtonState() {
  return _button->getDebounceButtonState();
}

int Heat::getControlType() {
  return _control->getControlType();
}

int Heat::nextControlCode() {
  return _control->nextControlCode();
}

int Heat::getControlCode() {
  return _control->getControlCode();
}

char *Heat::getHeader() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  strcpy(buffer, "Heat Control");
  return buffer;
}

char *Heat::getControlText() {
  char * buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case heatOff:
    strcpy(buffer, "Off");
    return buffer;
    break;
  case heatFurnace:
    strcpy(buffer, "Furnace");
    return buffer;
    break;
  case heatSolar:
    strcpy(buffer, "Solar");
    return buffer;
    break;
  case heatBoth:
    strcpy(buffer, "Both");
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int Heat::getLastResult() {
  return _lastResult;
}
