/*
  Heat.h - A library for controlling the pool furnace and Solar heating
  Created by Ken Rumer, May 13, 2014.
  Released into the public domain.
*/
#ifndef Heat_h
#define Heat_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"

enum HeatModes {
  heatOff,
  heatFurnace,
  heatSolar,
  heatBoth
};

class Heat
{
  public:
    Heat(int furnacePin1, int furnacePin2, int solarPin, int buttonPin,
         int controlType, MuxShield *muxShield);
    int setHeatMode(int heatMode);
    int getHeatMode();
    int controlDevice();
    void setProcessing(int controlCode);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalOutRow;
    int _muxShieldDigitalInRow;
    int _countControlCodes;
    HeatModes _initialHeatMode;
    MyRelay *_furnaceRelay1;
    MyRelay *_furnaceRelay2;
    MyRelay *_solarRelay;
    MyButton *_button;
    MyControl *_control;
    HeatModes _heatMode;
};
#endif
