#include "Arduino.h"
#include "MuxShield.h"
#include "math.h"
#include "MyThermistor.h"

MyThermistor::MyThermistor(int thermistorPin, MuxShield *muxShield, int muxShieldRow) {
  _useMuxShield = false;
  _muxShieldRow = 2;
  _thermistorPin = thermistorPin;
  if (muxShield) {
    _muxShield = muxShield;
    _muxShieldRow = muxShieldRow;
    _useMuxShield = true;
  }
}

int MyThermistor::getThermistorValue() {
  if (_useMuxShield) {
    return _muxShield->analogReadMS(_muxShieldRow, _thermistorPin);
  } else {
    return analogRead(_thermistorPin);
  }
}

double MyThermistor::getCTemperature() {
  return _convertRawADC(getThermistorValue());
}

double MyThermistor::getFTemperature() {
  return (_convertRawADC(getThermistorValue()) * 1.8) + 32.0;
}

double MyThermistor::_convertRawADC(int thermistorValue) {
  double temp;
  // See http://en.wikipedia.org/wiki/Thermistor for explanation of formula
  temp = log(((10240000/thermistorValue) - 10000));
  temp = 1 / (0.001129148 + (0.000234125 * temp) + (0.0000000876741 * temp * temp * temp));
  temp = temp - 273.15;           // Convert Kelvin to Celcius
  return temp;
}
