#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "Pump.h"

Pump::Pump(int pumpINP1Pin, int pumpINP2Pin, int pumpINP3Pin, int pumpINP4Pin,
           int buttonPin, int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalOutRow = 1;
  _muxShieldDigitalInRow = 2;
  _countControlCodes = 5;
  _initialPumpMode = pumpOff;
  _pumpINP1Relay = new MyRelay(pumpINP1Pin, muxShield, _muxShieldDigitalOutRow);
  _pumpINP2Relay = new MyRelay(pumpINP2Pin, muxShield, _muxShieldDigitalOutRow);
  _pumpINP3Relay = new MyRelay(pumpINP3Pin, muxShield, _muxShieldDigitalOutRow);
  _pumpINP4Relay = new MyRelay(pumpINP4Pin, muxShield, _muxShieldDigitalOutRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setPumpMode(_initialPumpMode);
}

int Pump::setPumpMode(int pumpMode) {
  switch (pumpMode) {
  case pumpOff:
    _pumpINP1Relay->setRelay(HIGH);
    _pumpINP2Relay->setRelay(HIGH);
    _pumpINP3Relay->setRelay(HIGH);
    _pumpINP4Relay->setRelay(HIGH);
    break;
  case pumpLow:
    _pumpINP1Relay->setRelay(LOW);
    _pumpINP2Relay->setRelay(HIGH);
    _pumpINP3Relay->setRelay(HIGH);
    _pumpINP4Relay->setRelay(LOW);
    break;
  case pumpMedium:
    _pumpINP1Relay->setRelay(HIGH);
    _pumpINP2Relay->setRelay(LOW);
    _pumpINP3Relay->setRelay(HIGH);
    _pumpINP4Relay->setRelay(LOW);
    break;
  case pumpHigh:
    _pumpINP1Relay->setRelay(LOW);
    _pumpINP2Relay->setRelay(LOW);
    _pumpINP3Relay->setRelay(HIGH);
    _pumpINP4Relay->setRelay(LOW);
    break;
  case pumpHighest:
    _pumpINP1Relay->setRelay(HIGH);
    _pumpINP2Relay->setRelay(HIGH);
    _pumpINP3Relay->setRelay(LOW);
    _pumpINP4Relay->setRelay(LOW);
    break;
  default:
    return 1;
  }
  _control->setControlCode(pumpMode);
  _pumpMode = (PumpModes)pumpMode;
  return 0;
}

int Pump::getPumpMode() {
  return _pumpMode;
}

int Pump::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setPumpMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void Pump::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int Pump::getProcessing() {
  return _control->getProcessing();
}

int Pump::getButtonState() {
  return _button->getDebounceButtonState();
}

int Pump::getControlType() {
  return _control->getControlType();
}

int Pump::nextControlCode() {
  return _control->nextControlCode();
}

int Pump::getControlCode() {
  return _control->getControlCode();
}

char *Pump::getHeader() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  strcpy(buffer, "Pump Control");
  return buffer;
}

char *Pump::getControlText() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case pumpOff:
    strcpy(buffer, "Off");
    return buffer;
    break;
  case pumpLow:
    strcpy(buffer, "Low");
    return buffer;
    break;
  case pumpMedium:
    strcpy(buffer, "Medium");
    return buffer;
    break;
  case pumpHigh:
    strcpy(buffer, "High");
    return buffer;
    break;
  case pumpHighest:
    strcpy(buffer, "Highest");
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int Pump::getLastResult() {
  return _lastResult;
}
