/*
  Pump.h - A library for controlling the speed of the pump
  Created by Ken Rumer, May 13, 2014.
  Released into the public domain.

//Pump speed table
// speed    INP1     INP2     INP3
//  1    -  Off   -  Off   -  Off
//  2    -  On    -  Off   -  Off
//  3    -  Off   -  On    -  Off
//  4    -  On    -  On    -  Off
//  5    -  Off   -  Off   -  On
//  6    -  Off   -  On    -  On
//  7    -  On    -  Off   -  On
//  8    -  On    -  On    -  On
// ----------------
// on/off INP4

*/
#ifndef Pump_h
#define Pump_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"

enum PumpModes {
  pumpOff,
  pumpLow,
  pumpMedium,
  pumpHigh,
  pumpHighest
};

class Pump
{
  public:
    Pump(int pumpINP1Pin, int pumpINP2Pin, int pumpINP3Pin, int pumpINP4Pin,
         int buttonPin, int controlType, MuxShield *muxShield);
    int setPumpMode(int pumpMode);
    int getPumpMode();
    int controlDevice();
    void setProcessing(int controlCode);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalOutRow;
    int _muxShieldDigitalInRow;
    int _countControlCodes;
    PumpModes _initialPumpMode;
    MyRelay *_pumpINP1Relay;
    MyRelay *_pumpINP2Relay;
    MyRelay *_pumpINP3Relay;
    MyRelay *_pumpINP4Relay;
    MyButton *_button;
    MyControl *_control;
    PumpModes _pumpMode;
};
#endif
