#include "Arduino.h"
#include "MuxShield.h"
#include "MyThermistor.h"
#include "MyControl.h"
#include "Temperature.h"

Temperature::Temperature(int airTempPin, int flowTempPin, int solarTempPin,
           int buttonPin, int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalInRow = 2;
  _muxShieldAnalogInRow = 3;
  _countControlCodes = 3;
  _initialTemperatureMode = temperatureAir;
  _initialTemperatureModeMode = temperatureModeFahrenheit;
  _airThermistor = new MyThermistor(airTempPin, muxShield, _muxShieldAnalogInRow);
  _flowThermistor = new MyThermistor(flowTempPin, muxShield, _muxShieldAnalogInRow);
  _solarThermistor = new MyThermistor(solarTempPin, muxShield, _muxShieldAnalogInRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setTemperatureModeMode(_initialTemperatureModeMode);
  setTemperatureMode(_initialTemperatureMode);
}

int Temperature::setTemperatureModeMode(int temperatureModeMode) {
  switch (temperatureModeMode) {
  case temperatureModeCelsius:
    break;
  case temperatureModeFahrenheit:
    break;
  default:
    return 1;
  }
  _temperatureModeMode = (TemperatureModeModes)temperatureModeMode;
  return 0;
}

int Temperature::getTemperatureModeMode() {
  return _temperatureModeMode;
}

int Temperature::setTemperatureMode(int temperatureMode) {
  switch (temperatureMode) {
  case temperatureAir:
    break;
  case temperatureFlow:
    break;
  case temperatureSolar:
    break;
  default:
    return 1;
  }
  _control->setControlCode(temperatureMode);
  _temperatureMode = (TemperatureModes)temperatureMode;
  return 0;
}

int Temperature::getTemperatureMode() {
  return _temperatureMode;
}

int Temperature::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setTemperatureMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void Temperature::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int Temperature::getProcessing() {
  return _control->getProcessing();
}

int Temperature::getButtonState() {
  return _button->getDebounceButtonState();
}

int Temperature::getControlType() {
  return _control->getControlType();
}

int Temperature::nextControlCode() {
  return _control->nextControlCode();
}

int Temperature::getControlCode() {
  return _control->getControlCode();
}

char *Temperature::getHeader() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case temperatureAir:
    strcpy(buffer, "Air Temp");
    return buffer;
    break;
  case temperatureFlow:
    strcpy(buffer, "Flow Temp");
    return buffer;
    break;
  case temperatureSolar:
    strcpy(buffer, "Solar Temp");
    return buffer;
    break;
  default:
    strcpy(buffer, "Temp");
    return buffer;
    break;
  }
}

char *Temperature::getControlText() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case temperatureAir:
    switch (getTemperatureModeMode()) {
    case temperatureModeCelsius:
      sprintf(buffer, "%.2d C\0", int(_airThermistor->getCTemperature()));
      break;
    case temperatureModeFahrenheit:
      sprintf(buffer, "%.2d F\0", int(_airThermistor->getFTemperature()));
      break;
    }
    return buffer;
    break;
  case temperatureFlow:
    switch (getTemperatureModeMode()) {
    case temperatureModeCelsius:
      sprintf(buffer, "%.0d C", int(_flowThermistor->getCTemperature()));
      break;
    case temperatureModeFahrenheit:
      sprintf(buffer, "%.0d F", int(_flowThermistor->getFTemperature()));
      break;
    }
    return buffer;
    break;
  case temperatureSolar:
    switch (getTemperatureModeMode()) {
    case temperatureModeCelsius:
      sprintf(buffer, "%.0d C", int(_solarThermistor->getCTemperature()));
      break;
    case temperatureModeFahrenheit:
      sprintf(buffer, "%.0d F", int(_solarThermistor->getFTemperature()));
      break;
    }
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int Temperature::getLastResult() {
  return _lastResult;
}
