/*
  Temperature.h - A library for getting the temperatures of the pool
  Created by Ken Rumer, July 5, 2014.
  Released into the public domain.
*/
#ifndef Temperature_h
#define Temperature_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyThermistor.h"
#include "MyControl.h"
#include "MyButton.h"

enum TemperatureModes {
  temperatureAir,
  temperatureFlow,
  temperatureSolar
};

enum TemperatureModeModes {
  temperatureModeCelsius,
  temperatureModeFahrenheit
};

class Temperature
{
  public:
    Temperature(int airTempPin, int flowTempPin, int solarTempPin,
         int buttonPin, int controlType, MuxShield *muxShield);
    int setTemperatureMode(int temperatureMode);
    int getTemperatureMode();
    int setTemperatureModeMode(int temperatureModeMode);
    int getTemperatureModeMode();
    int controlDevice();
    void setProcessing(int controlCode);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    double getAirFTemperature();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalInRow;
    int _muxShieldAnalogInRow;
    int _countControlCodes;
    TemperatureModes _initialTemperatureMode;
    TemperatureModeModes _initialTemperatureModeMode;
    MyThermistor *_airThermistor;
    MyThermistor *_flowThermistor;
    MyThermistor *_solarThermistor;
    MyButton *_button;
    MyControl *_control;
    TemperatureModes _temperatureMode;
    TemperatureModeModes _temperatureModeMode;
};
#endif
