#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"
#include "WaterLight.h"

WaterLight::WaterLight(int poolLightPin, int spaLightPin, int buttonPin,
                       int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalOutRow = 1;
  _muxShieldDigitalInRow = 2;
  _countControlCodes = 4;
  _initialWaterLightMode = waterLightOff;
  _poolLightRelay = new MyRelay(poolLightPin, muxShield, _muxShieldDigitalOutRow);
  _spaLightRelay = new MyRelay(spaLightPin, muxShield, _muxShieldDigitalOutRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setWaterLightMode(_initialWaterLightMode);
}

int WaterLight::setWaterLightMode(int waterLightMode) {
  switch (waterLightMode) {
  case waterLightOff:
    _poolLightRelay->setRelay(HIGH);
    _spaLightRelay->setRelay(HIGH);
    break;
  case waterLightPool:
    _poolLightRelay->setRelay(LOW);
    _spaLightRelay->setRelay(HIGH);
    break;
  case waterLightSpa:
    _poolLightRelay->setRelay(HIGH);
    _spaLightRelay->setRelay(LOW);
    break;
  case waterLightBoth:
    _poolLightRelay->setRelay(LOW);
    _spaLightRelay->setRelay(LOW);
    break;
  default:
    return 1;
  }
  _control->setControlCode(waterLightMode);
  _waterLightMode = (WaterLightModes)waterLightMode;
  return 0;
}

int WaterLight::getWaterLightMode() {
  return _waterLightMode;
}

int WaterLight::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setWaterLightMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void WaterLight::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int WaterLight::getProcessing() {
  return _control->getProcessing();
}

int WaterLight::getButtonState() {
  return _button->getDebounceButtonState();
}

int WaterLight::getControlType() {
  return _control->getControlType();
}

int WaterLight::nextControlCode() {
  return _control->nextControlCode();
}

int WaterLight::getControlCode() {
  return _control->getControlCode();
}

char *WaterLight::getHeader() {
  char * buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  strcpy(buffer, "Water Light Ctrl");
  return buffer;
}

char *WaterLight::getControlText() {
  char *buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case waterLightOff:
    strcpy(buffer, "Off");
    return buffer;
    break;
  case waterLightPool:
    strcpy(buffer, "Pool");
    return buffer;
    break;
  case waterLightSpa:
    strcpy(buffer, "Spa");
    return buffer;
    break;
  case waterLightBoth:
    strcpy(buffer, "Both");
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int WaterLight::getLastResult() {
  return _lastResult;
}
