/*
  WaterLight.h - A library for controlling the lights under water
  Created by Ken Rumer, May 13, 2014.
  Released into the public domain.
*/
#ifndef WaterLight_h
#define WaterLight_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"

enum WaterLightModes {
  waterLightOff,
  waterLightPool,
  waterLightSpa,
  waterLightBoth
};

class WaterLight
{
  public:
    WaterLight(int poolLightPin, int spaLightPin, int buttonPin,
               int controlType, MuxShield *muxShield);
    int setWaterLightMode(int waterLightMode);
    int getWaterLightMode();
    int controlDevice();
    void setProcessing(int controlCode);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalOutRow;
    int _muxShieldDigitalInRow;
    int _countControlCodes;
    WaterLightModes _initialWaterLightMode;
    MyRelay *_poolLightRelay;
    MyRelay *_spaLightRelay;
    MyButton *_button;
    MyControl *_control;
    WaterLightModes _waterLightMode;
};
#endif
