#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"
#include "YardLight.h"

YardLight::YardLight(int treeLightPin, int walkwayLightPin,
                     int buttonPin, int controlType, MuxShield *muxShield) {
  _lastResult = 0;
  _muxShieldDigitalOutRow = 1;
  _muxShieldDigitalInRow = 2;
  _countControlCodes = 4;
  _initialYardLightMode = yardLightOff;
  _treeLightRelay = new MyRelay(treeLightPin, muxShield, _muxShieldDigitalOutRow);
  _walkwayLightRelay = new MyRelay(walkwayLightPin, muxShield, _muxShieldDigitalOutRow);
  _button = new MyButton(buttonPin, muxShield, _muxShieldDigitalInRow);
  _control = new MyControl(controlType, _countControlCodes);
  setYardLightMode(_initialYardLightMode);
}

int YardLight::setYardLightMode(int yardLightMode) {
  switch (yardLightMode) {
  case yardLightOff:
    _treeLightRelay->setRelay(HIGH);
    _walkwayLightRelay->setRelay(HIGH);
    break;
  case yardLightTree:
    _treeLightRelay->setRelay(LOW);
    _walkwayLightRelay->setRelay(HIGH);
    break;
  case yardLightWalkway:
    _treeLightRelay->setRelay(HIGH);
    _walkwayLightRelay->setRelay(LOW);
    break;
  case yardLightAll:
    _treeLightRelay->setRelay(LOW);
    _walkwayLightRelay->setRelay(LOW);
    break;
  default:
    return 1;
  }
  _control->setControlCode(yardLightMode);
  _yardLightMode = (YardLightModes)yardLightMode;
  return 0;
}

int YardLight::controlDevice() {
  if (getProcessing()) {
    _control->setProcessing(false);
    _lastResult = setYardLightMode(getControlCode());
    return _lastResult;
  }
  _lastResult = -1;
  return _lastResult;
}

void YardLight::setProcessing(int controlCode) {
  _control->setProcessing(true);
  _control->setControlCode(controlCode);
}

int YardLight::getProcessing() {
  return _control->getProcessing();
}

int YardLight::getButtonState() {
  return _button->getDebounceButtonState();
}

int YardLight::getControlType() {
  return _control->getControlType();
}

int YardLight::nextControlCode() {
  return _control->nextControlCode();
}

int YardLight::getControlCode() {
  return _control->getControlCode();
}

char *YardLight::getHeader() {
  char * buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  strcpy(buffer, "Yard Light Ctrl");
  return buffer;
}

char *YardLight::getControlText() {
  char * buffer = (char *)malloc(17);
  memset(buffer, '\0', 17);
  switch (getControlCode()) {
  case yardLightOff:
    strcpy(buffer, "Off");
    return buffer;
    break;
  case yardLightTree:
    strcpy(buffer, "Tree");
    return buffer;
    break;
  case yardLightWalkway:
    strcpy(buffer, "Walk");
    return buffer;
    break;
  case yardLightAll:
    strcpy(buffer, "All");
    return buffer;
    break;
  default:
    strcpy(buffer, "Unknown");
    return buffer;
    break;
  }
}

int YardLight::getLastResult() {
  return _lastResult;
}
