/*
  YardLight.h - A library for controlling the lights in the backyard
  Created by Ken Rumer, May 13, 2014.
  Released into the public domain.
*/
#ifndef YardLight_h
#define YardLight_h

#include "Arduino.h"
#include "MuxShield.h"
#include "MyRelay.h"
#include "MyControl.h"
#include "MyButton.h"

enum YardLightModes {
  yardLightOff,
  yardLightTree,
  yardLightWalkway,
  yardLightAll
};

class YardLight
{
  public:
    YardLight(int treeLightPin, int walkwayLightPin, 
              int buttonPin, int controlType, MuxShield *muxShield);
    int setYardLightMode(int yardLightMode);
    int getYardLightMode();
    int controlDevice();
    void setProcessing(int controlCode);
    int getProcessing();
    int getButtonState();
    int getControlType();
    int nextControlCode();
    int getControlCode();
    char *getHeader();
    char *getControlText();
    int getLastResult();
  private:
    int _lastResult;
    int _muxShieldDigitalOutRow;
    int _muxShieldDigitalInRow;
    int _countControlCodes;
    YardLightModes _initialYardLightMode;
    MyRelay *_treeLightRelay;
    MyRelay *_walkwayLightRelay;
    MyButton *_button;
    MyControl *_control;
    YardLightModes _yardLightMode;
};
#endif
